from datetime import datetime
from enum import Enum
from pydantic import BaseModel
from sqlmodel import Field
from typing import Literal, Optional

from .base_models import BaseSQLModel
from .shared_enums import TaskType

"""
Fix this if possible: All class have the same mypy error:
Metaclass conflict:
the metaclass of a derived class must be a (non-strict) subclass of the metaclasses of all its bases  [misc]
"""


class StatusType(str, Enum):
    NOT_STARTED = "NOT_STARTED"
    IN_PROGRESS = "IN_PROGRESS"
    COMPLETED = "COMPLETED"
    # ERROR = "ERROR"


class AssigneeType(str, Enum):
    CODER = "CODER"
    CTO = "CTO"
    CEO = "CEO"
    COO = "COO"


class NecessityType(str, Enum):
    """Must/Should/Could/Won't Have"""

    MUST = "MUST"
    SHOULD = "SHOULD"
    COULD = "COULD"
    WONT = "WONT"


class PriorityType(str, Enum):
    HIGH = "HIGH"
    MEDIUM = "MEDIUM"
    LOW = "LOW"


class BaseProjectModel(BaseSQLModel):
    description: str
    status: StatusType
    name: str

    start_date: Optional[datetime] = None
    end_date: Optional[datetime] = None

    # Support these fields in the future
    # estimated_duration: Optional[str]
    # deadline: Optional[datetime]

    # TODO: add validator for start_date and end_date and status


class Project(BaseProjectModel, table=True):
    pass


class Objective(BaseProjectModel, table=True):
    project_id: int = Field(foreign_key="project.id")


class KeyResult(BaseProjectModel, table=True):
    objective_id: int = Field(foreign_key="objective.id")


class Requirement(BaseProjectModel, table=True):
    key_result_id: int = Field(foreign_key="keyresult.id")
    necessity: NecessityType


class ScopedTask(BaseProjectModel, table=True):
    requirement_id: int = Field(foreign_key="requirement.id")
    assignee: Optional[AssigneeType] = None
    priority: Optional[PriorityType] = None
    task_type: Optional[TaskType] = TaskType.UNDEFINED

    # Additional fields to provide more information
    # estimated_duration: Optional[str]
    # difficulty: Optional[str]

    # This can be used to link the task to Gitlab MR
    # merge_request_id: Optional[str] = None


class ScopedTaskStep(BaseSQLModel, table=True):
    task_id: int = Field(foreign_key="scopedtask.id")
    action: str
    status: StatusType
    expected_result: Optional[str] = None
    # consider adding execution_log here


# Roadmap are pydantic models and not SQL models. It uses files to store and directly edit.
# It can be converted to SQL model if needed.


class RoadmapMilestone(BaseModel):
    title: str
    description: str
    date: datetime
    type: Literal["HISTORICAL", "FUTURE"]
    category: Literal[
        "KEY-METRICS",
        "TRADING-ENGINE",
        "ML-MODEL",
        "INFRASTRUCTURE",
        "AI-TEAM",
        "OPERATIONS",
        "HIRING",
    ]


class Roadmap(BaseModel):
    name: str
    description: str
    milestones: list[RoadmapMilestone] = []

    @property
    def historical_milestones(self) -> list[RoadmapMilestone]:
        return [m for m in self.milestones if m.type == "HISTORICAL"]

    @property
    def future_milestones(self) -> list[RoadmapMilestone]:
        return [m for m in self.milestones if m.type == "FUTURE"]


# These are view models which are not stored in the database. They are used to specify AI response and their display.


class OKR(BaseModel):
    objective: Objective
    key_results: list[KeyResult]


class KeyResultRequirements(BaseModel):
    key_result: KeyResult
    requirements: list[Requirement]


class ScopedTaskStepList(BaseModel):
    task_id: int
    steps: list[ScopedTaskStep]



import sqlalchemy
from pydantic import BaseModel
from sqlmodel import Field
from typing import Optional

from .base_models import BaseSQLModel
from .project_models import KeyResult, Objective, Project, Requirement, ScopedTask


class TaskResponseFormat(BaseModel):
    data_model_class: str
    data_model_schema: Optional[dict] = None
    data_model_example: Optional[dict] = None


class ProjectContext(BaseModel):
    project: Project
    objective: Objective
    key_result: KeyResult
    requirement: Requirement
    scoped_task: ScopedTask


class FileContext(BaseModel):
    file_uri: str
    file_content: Optional[str] = None


class TaskContext(BaseSQLModel, table=True):
    response_format: Optional[TaskResponseFormat] = Field(
        default=None, sa_type=sqlalchemy.JSON
    )
    project_context: Optional[ProjectContext] = Field(
        default=None, sa_type=sqlalchemy.JSON
    )
    input_files: list[FileContext] = Field(sa_type=sqlalchemy.JSON)
    output_files: list[str] = Field(sa_type=sqlalchemy.JSON)

    ai_task_id: Optional[int] = Field(default=None, foreign_key="aitask.id")
    scoped_task_id: Optional[int] = Field(default=None, foreign_key="scopedtask.id")



from enum import Enum
from sqlmodel import Field, Relationship
from typing import Optional

from .base_models import BaseSQLModel
from .context_models import TaskContext
from .message_models import TaskMessage
from .shared_enums import ModelNameLLM


class AiTaskStatus(str, Enum):
    PENDING = "PENDING"
    IN_PROGRESS = "IN_PROGRESS"
    COMPLETED = "COMPLETED"
    FAILED = "FAILED"


class AiTask(BaseSQLModel, table=True):
    llm_model_name: ModelNameLLM
    prompt_text: str
    status: AiTaskStatus

    user_prompt_id: int = Field(foreign_key="userprompt.id")
    scoped_task_id: Optional[int] = Field(foreign_key="scopedtask.id")

    context: Optional[TaskContext] = Relationship(
        sa_relationship_kwargs={"lazy": "selectin"}
    )
    messages: list[TaskMessage] = Relationship(
        sa_relationship_kwargs={"lazy": "selectin"}
    )


# Do not persist AiCompletion until: the use case is clear and AiTask table isn't enough
class AiCompletion(BaseSQLModel):
    success: bool
    text_output: Optional[str] = None
    json_output: Optional[dict] = None
    # add more fields for output display

    user_prompt_id: Optional[int] = None



from datetime import datetime
from sqlmodel import Field, SQLModel
from typing import Optional

# Consider the correct usage of extend_existing for different environments
should_extend_existing = True


class BaseSQLModel(SQLModel):
    """
    BaseSQLModel is the base class for all SQL models
    - SQLModel is a thin wrapper around Pydantic and SQLAlchemy
    - Subclass of SQLModel can use any functionality from Pydantic and SQLAlchemy
    """

    __table_args__ = {"extend_existing": should_extend_existing}
    id: Optional[int] = Field(default=None, primary_key=True)
    # make this required in the future
    created_at: Optional[datetime] = Field(default_factory=datetime.utcnow)



from enum import Enum
from typing import Optional

from .base_models import BaseSQLModel


# TODO: use this to pass test; run migration after we are ready to persist the changes in DB


class FileChangeType(str, Enum):
    CREATE = "CREATE"
    UPDATE = "UPDATE"
    DELETE = "DELETE"


class FileChange(BaseSQLModel):
    change_type: FileChangeType
    file_uri: str
    file_content: Optional[str] = None


class AiFileChanges(BaseSQLModel):
    summary: str
    file_changes: list[FileChange]


# TODO: consider the association of AiFileChanges, Testing and AiTask better; models below are just a draft


class FileChangeStatus(str, Enum):
    PENDING = "PENDING"
    APPLIED = "APPLIED"
    TESTED = "TESTED"
    REVERTED = "REVERTED"
    COMMITTED = "COMMITTED"


class FileChangesTest(BaseSQLModel):
    test_type: str
    test_passed: bool
    failure_message: Optional[str] = None
    test_cases: Optional[dict] = None


class FileChangesResult(BaseSQLModel):
    status: FileChangeStatus
    summary: str
    file_changes: list[FileChange]
    tests: list[FileChangesTest]
    # ai_file_changes_id: Optional[int] = Field(default=None, foreign_key="aifilechanges.id")
    # ai_task_id: Optional[int] = Field(default=None, foreign_key="aitask.id")



from enum import Enum
from sqlmodel import Field
from typing import Optional

from .base_models import BaseSQLModel


class MessageRole(str, Enum):
    USER = "user"
    ASSISTANT = "assistant"


class TaskMessage(BaseSQLModel, table=True):
    content: str
    role: MessageRole

    ai_task_id: Optional[int] = Field(default=None, foreign_key="aitask.id")


# TODO: move message models:
#  - consider how to consolidate StandardMessage and TaskMessage
#  - consider using a single message in response

# class StandardMessage(BaseModel):
#     content: str
#     role: Literal["user", "assistant"]
#
#
# class CreateMessageRequest(BaseModel):
#     model: ModelNameLLM
#     messages: list[StandardMessage]
#     json_mode: bool = False
#     system_message: Optional[str] = None
#
#
# class CreateMessageResponse(BaseModel):
#     messages: list[StandardMessage]
#     json_dict: Optional[dict] = None



from enum import Enum

from .base_models import BaseSQLModel


class UserRole(str, Enum):
    ADMIN = "ADMIN"
    TEST = "TEST"


class User(BaseSQLModel, table=True):
    name: str
    role: UserRole



from enum import Enum


class AppEnv(str, Enum):
    DEV = "dev"
    TEST = "test"
    STAG = "stag"
    PROD = "prod"


class ModelNameLLM(str, Enum):
    CLAUDE_3_5_SONNET = "claude-3-5-sonnet-20240620"
    CLAUDE_3_HAIKU = "claude-3-haiku-20240307"
    CLAUDE_3_SONNET = "claude-3-sonnet-20240229"
    CLAUDE_3_OPUS = "claude-3-opus-20240229"
    GPT_4 = "gpt-4o"
    GPT_3 = "gpt-3.5-turbo"


class TaskType(str, Enum):
    # Coder
    FRONTEND = "FRONTEND"
    SVC_IMPL = "SVC_IMPL"
    UNIT_TEST = "UNIT_TEST"
    INTERFACE = "INTERFACE"
    DATA_MODEL = "DATA_MODEL"

    # CTO
    SCOPED_TASK = "SCOPED_TASK"
    REQUIREMENT = "REQUIREMENT"
    ARCHITECTURE = "ARCHITECTURE"
    INFRASTRUCTURE = "INFRASTRUCTURE"
    PROTOTYPE = "PROTOTYPE"

    # CEO
    ROADMAP = "ROADMAP"
    OKR = "OKR"

    # COO
    COMPLIANCE = "COMPLIANCE"

    # General
    DOCUMENTATION = "DOCUMENTATION"

    # Special
    UNDEFINED = "UNDEFINED"



from typing import Optional

import sqlalchemy
from pydantic import BaseModel
from sqlmodel import Field

from .base_models import BaseSQLModel
from .shared_enums import ModelNameLLM, TaskType


class UserPrompt(BaseSQLModel, table=True):
    prompt_text: str
    response_class_name: Optional[str] = None

    # This should support module notation
    input_files: list[str] = Field(sa_type=sqlalchemy.JSON)
    output_files: list[str] = Field(sa_type=sqlalchemy.JSON)
    # Support data change later

    # LLM Model Config
    llm_model_name: ModelNameLLM
    # add Temperature, etc

    # add user_id; consider required vs optional, creation timing and place
    # user_id: Optional[int] = Field(default=None, foreign_key="user.id")
    scoped_task_id: Optional[int] = Field(default=None, foreign_key="scopedtask.id")


class PromptTemplate(BaseModel):
    task_type: TaskType
    scope: str
    response_class_name: str
    general_instruction: str
    scoped_instruction: str
    input_files: list[str]
    output_files: list[str]


class PromptTemplateSimple(BaseModel):
    response_class_name: str
    task_description: str
    extra_instructions: list[str]
    input_files: list[str]
